/*
  Name: Animation editor 
  Copyright: The Trivium 2005 www.thetrivium.org
  Author: Vercingetorix (pauldls@hotmail.com)
  Date: 1/14/05
  Version: 0.49b
  Description: program to edit the pack.dat and skeleton.dat (edit animations) 
*/

/* 
Notes:
    If you wish you use this code or any part of it please email me first before
    you use it in any public program release.
    
Update log:
    -1/18/05   -
    Fixed bug in scale function that scaled mounts incorrectly
    -1/14/05   -
    Added scale function
    Now when adding a new skeleton the fn copys from the pak not the disk
    -12/20/04  -
    Finished movement speed function. Made filename typing a little more friendly.
    -12/16/04  -
    Off by one error in remove function fixed.
    -12/16/04  -
    Added changing unit speed function. (MovementManip class)
    TODO: Change not just transit but actual Positions
    - 12/9/04  -
    Fixed bug in remove function that failed to update the data offsets (d'oh!)
    - 12/6/04  -
    Fixed trailing spaces at the end of filenames
    - 11/21/04 - 
    Added skeleton support
    - 11/20/04 - 
    Extended command line arguments to allow for a "list" of files.
    - 11/19/04 -
    Wrote most of the rest. Realeased to public.
    TODO: add support for new skeletons
    - 11/18/04 - 
    Wrote remove function. A little tricky, 
    I can't believe there is no way to remove a chunk of data painlessly, should write fn...
    Wrote command line arguments
    Wrote add function
    TODO: be able to work with entire directories instead of single files
    - 11/16/04 -
    Wrote base code and prompting. Also wrote helper functions
    TODO: Write CMenuItem execute functions that actually edit the dat file
    - 11/10/04 -
    Started laying class foundations    
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <ctype.h> 
#include <direct.h>



#define NUMITEMS 7
#define MENU_ADD 0
#define MENU_REMOVE 1
#define MENU_REPLACE 2
#define MENU_HELP 3
#define MENU_NEWSKEL 4
#define MENU_MOVEMANIP 5
#define MENU_SCALE 6

#define MENU_QUIT 8

#define toupper(c) (c >= 'a' && c <= 'z') ? c -'a'+'A' : c;  


int getfilesize(FILE *fp){
    int curpos = ftell(fp);
    fseek(fp,0,SEEK_END); 
    int fsize=ftell(fp); 
    fseek(fp,curpos,SEEK_SET); 
    return fsize;
    }
/*Some case-insenseitve functions*/
char* strupper(char* str){

    for(int i=0; i<strlen(str); i++)
	str[i] = toupper(str[i]);
	return str;
}

char* stristr(const char* str, const char* sstr){
	char ustr[512] = {0};
	char usstr[512] = {0};

	strncpy(ustr,str,511); strupper(ustr);
	strncpy(usstr,sstr,511); strupper(usstr);

	return strstr(ustr,usstr) ? (char*)&str[ustr - strstr(ustr,usstr)] : NULL;

}

/*lol found out these 2 fns already exist...*/
int stricmp ( const char * string1, const char * string2 ){
	char ustr1[512] = {0};
	char ustr2[512] = {0};

	strncpy(ustr1,string1,511); strupper(ustr1);
	strncpy(ustr2,string2,511); strupper(ustr2);

	return strcmp(ustr1,ustr2);

}

int strnicmp ( const char * string1, const char * string2, size_t num ){
	char ustr1[512] = {0};
	char ustr2[512] = {0};

	strncpy(ustr1,string1,511); strupper(ustr1);
	strncpy(ustr2,string2,511); strupper(ustr2);

	return strncmp(ustr1,ustr2,num);

}

inline char* CleanTrailing(char *str){
	char *pstr=&str[strlen(str)];
	while(strchr(--pstr,' ')) *strchr(pstr,' ')=0;
	return str;
}

bool CreateBlankFile(char * fn){
    FILE *fp = fopen(fn,"wb");
    if(!fp) return false;
    char blank[16]={0};
    fwrite(blank,12,1,fp);
    fclose(fp);
    return true;
}    

/*Global file pointers...*/
char pcwd[_MAX_PATH]={0};  
FILE* idx_fp = fopen("pack.idx","r+b"); 
FILE* dat_fp = fopen("pack.dat","r+b");

FILE* skl_if = fopen("skeletons.idx","r+b"); 
FILE* skl_df = fopen("skeletons.dat","r+b");

/*Class defs*/

class skl_Entry{
    public:
         skl_Entry(unsigned int a=0,unsigned int b=0,unsigned int c=0,const char *str=NULL){
             strlength=a;
             offset = b;
             fsize=c;
             if(str) strcpy(fname,str); else *fname = NULL;
             }
         
    public:
        bool Read(FILE *fp){
            if(!fp) return false;
            fread(this,12,1,fp);
            fread(fname,strlength,1,fp);
            return true;
        }
        bool Write(FILE *fp){
            if(!fp) return false;
            fwrite(this,12,1,fp); 
            fwrite(fname,strlength,1,fp);
            return true;
        }        
    public:
        int strlength;
        unsigned int offset;
        unsigned int fsize;
        char fname[256];
             
};    
class idx_Entry{
    public:
        idx_Entry(int a=0,int b=0,int c=0,float d=0.0,short e=0,short f=0,const char*g=NULL){
            unkownInt=a;
            dataOffset=b;
            dataSize=c;
            scale=d;
            numFrames=e;
            numBones=f;
            if(g) strcpy(fname,g);
        }    
       bool Read(FILE* fp){
            
            fread(this,16,1,fp);;
            fread(&numFrames,2,1,fp);
            fread(&numBones,2,1,fp);
            fseek(fp,1,SEEK_CUR);
            for(int i=0; !i? true : fname[ i-1 ]!='\0'; i++) fread(&fname[i],1,1,fp);
            return true;
            }
       bool Write(FILE* fp){
            
            fwrite(this,20,1,fp);
            //fwrite(&numFrames,2,1,fp);
            //fwrite(&numBones,2,1,fp);
            char one=1;
            fwrite(&one,1,1,fp);
            fwrite(fname,strlen(fname)+1,1,fp);
            return true;
        }    
            
    public:
        int unkownInt; 
        unsigned long dataOffset;
        unsigned long dataSize;
        float scale;
        short numFrames;
        short numBones;
        char fname[256];
        
            
};   

class CMenuItem{
    public:
        virtual void execute(const char*){};
    protected:  
        /*helper functions*/
        void decOffsets(int start, unsigned int amt){
            fseek(idx_fp,0,SEEK_SET);
            int nentrys = getnfiles()-1;
            fseek(idx_fp,20,SEEK_SET);
            //FILE *tfp = fopen("tidx","wb");
            //printf("Entrys: %d",nentrys);
            for(int i=0; i<nentrys; i++){
                idx_Entry tent;
                int curpos = ftell(idx_fp);
                tent.Read(idx_fp);
                if(i>=start){
                    //printf("%s\n",tent.fname);
                    tent.dataOffset -=amt;
                    fseek(idx_fp,curpos,SEEK_SET);
                    if(!tent.Write(idx_fp)) fputs("idx dec offset write fail\n",stderr);
                    fflush(idx_fp);
                }    
            } 
           // fclose(tfp);         
        }    
        int getnfiles(bool skl=false){
            int numfiles=0;
            fpos_t curpos = fgetpos(skl ? skl_if : idx_fp,&curpos);
            fseek(skl ? skl_if : idx_fp,16,SEEK_SET);
            fread(&numfiles,4,1,skl ? skl_if : idx_fp);
            fsetpos(skl ? skl_if : idx_fp,&curpos);
            return numfiles;
        }
        
        /*increments numfiles*/
    void incfiles(bool skl=false){
        int numfiles=getnfiles(skl);
        numfiles++;
        
        int curpos = ftell(skl ? skl_if : idx_fp);
        fseek(skl ? skl_if : idx_fp,16,SEEK_SET);
        fwrite(&numfiles,4,1,skl ? skl_if : idx_fp);
        fseek(skl ? skl_if : idx_fp,curpos,SEEK_SET);
        
        curpos = ftell(skl ? skl_df : dat_fp);
        fseek(skl ? skl_df : dat_fp,16,SEEK_SET);
        fwrite(&numfiles,4,1,skl ? skl_df : dat_fp);
        fseek(skl ? skl_df : dat_fp,curpos,SEEK_SET);
        }
        
        /*decrements numfiles*/
    void decfiles(bool skl=false){
        int numfiles=getnfiles();
        numfiles--;
        
        int curpos = ftell(skl ? skl_if : idx_fp);
        fseek(skl ? skl_if : idx_fp,16,SEEK_SET);
        fwrite(&numfiles,4,1,skl ? skl_if : idx_fp);
        fseek(skl ? skl_if : idx_fp,curpos,SEEK_SET);
        
        curpos = ftell(skl ? skl_df : dat_fp); 
        fseek(skl ? skl_df : dat_fp,16,SEEK_SET);
        fwrite(&numfiles,4,1,skl ? skl_df : dat_fp);
        fseek(skl ? skl_df : dat_fp,curpos,SEEK_SET);          
             
        
        }
        
    int  findfile(const char* fname){/*searches for a file by name*/
        char tmpstr[256]={0};
        int filenum=0;
        char*pstr = tmpstr;
        bool found=false;
        
        fseek(idx_fp,41,SEEK_SET); /*first string*/
        while(!found && !feof(idx_fp)){
                for(int i=0; !i? 1 : tmpstr[ i-1 ]!='\0'; i++) fread(&tmpstr[i],1,1,idx_fp); //booyah!
                /*
                if(strchr(tmpstr,'/')) pstr = strrchr(tmpstr,'/')+1;
                printf("%s\n",pstr);
                *///was thinking about just searching for the filename but nah full path now
                if(!strnicmp(pstr,fname,strlen(fname))) found=true;
                else filenum++;
                if(!feof(idx_fp)&&!found) fseek(idx_fp,21,SEEK_CUR);
                //printf("%s %d\n",pstr,ftell(idx_fp));
               
            }  
           // printf("%d %d %d \n",ftell(idx_fp),(strlen(pstr)+1+21),ftell(idx_fp)-(strlen(pstr)+1+21));
           if(found){ fseek(idx_fp,ftell(idx_fp)-(strlen(pstr)+1+21),SEEK_SET); printf("Found file at entry %d\n",filenum);  return filenum;}
           else {printf("Couldn't find file %s\n",fname); return -1;}
        }
        
        bool copy(char* fn, char *nfn){
            int filenum=-1;
            idx_Entry ent;
            
            if( (filenum=findfile(fn))==-1 ) return false;
            
            /*write idx entry*/
            ent.Read(idx_fp);
            idx_Entry entry(strlen(nfn)+10,getfilesize(dat_fp),ent.dataSize,ent.scale,ent.numFrames,ent.numBones,nfn);
            fseek(idx_fp,0,SEEK_END);
            entry.Write(idx_fp);
            
            /*write data*/
            char *bff = new char[ent.dataSize];
            fseek(dat_fp,ent.dataOffset,SEEK_SET);
            fread(bff,1,ent.dataSize,dat_fp);
            fseek(dat_fp,0,SEEK_END);
            fwrite(bff,1,ent.dataSize,dat_fp);
            delete [] bff;
            incfiles();
            
            fflush(idx_fp); fflush(dat_fp);
            return true;

        } 
           
    public:
        virtual void prompt(const char *pstr){
            memset(filename,0,256);
            printf("Enter %s: ", pstr );
            if(*gets(filename)==NULL) return;
            execute(filename);
            };
        virtual void prompt(){
            memset(filename,0,256);
            printf("Enter %s: ","filename");
            if(*gets(filename)==NULL) return;
            for(int i=0; i<strlen(filename); i++) if (filename[i]=='\\')  filename[i]='/';
            if(stristr(filename,"d/a/")){
                char *sstr = stristr(filename,"d/a/");
                char nstr[256]={0};
                strncpy(nstr,filename,sstr-filename);
                strcat(nstr,"data/animations/");
                strcat(nstr,sstr+strlen("d/a/"));
                strncpy(filename,nstr,255);
            }    
            execute(filename);
            };
        
    protected:
        char filename[256];
};

class IDXF_Remove: public CMenuItem{
    public:
        virtual void prompt(){printf("(Remove function)\n"); CMenuItem::prompt();}
        virtual void execute(const char*fname){
            int filenum = findfile(fname); 
            if(filenum!=-1){
               int curpos = ftell(idx_fp);
               char *bff = new char[curpos];
               char *bff2 = NULL;
               rewind(idx_fp);
               fread(bff,1,curpos,idx_fp);
               
               idx_Entry rmEntry;
               rmEntry.Read(idx_fp);
               
               int amtLeft = getfilesize(idx_fp) - ftell(idx_fp);
               
               if(amtLeft){
                   bff2 = new char[amtLeft];
                   fread(bff2,1,amtLeft,idx_fp);
               }    
               
               
               char idxPath[256]={0}; strcpy(idxPath,pcwd); strcat(idxPath,"\\pack.idx");
               freopen(idxPath,"w+b",idx_fp);
               
               fwrite(bff,1,curpos,idx_fp);
               if(bff2) fwrite(bff2,1,amtLeft,idx_fp);
               fflush(idx_fp);
               decOffsets(filenum,rmEntry.dataSize);
               
               freopen(idxPath,"r+b",idx_fp);
               decfiles();
               
               char *dbff= new char[rmEntry.dataOffset];
               rewind(dat_fp);
               fread(dbff,rmEntry.dataOffset,1,dat_fp);
               fseek(dat_fp,rmEntry.dataSize,SEEK_CUR);
               
               int amtLeft2 = getfilesize(dat_fp)-ftell(dat_fp);
               //printf("left2: %d %d\n",amtLeft2,ftell(dat_fp));
               char *dbff2= NULL;
               if(amtLeft2){
                   dbff2 = new char[amtLeft2];
                   fread(dbff2,1,amtLeft2,dat_fp);
               } 
                           
               
               char datPath[256]={0}; strcpy(datPath,pcwd); strcat(datPath,"\\pack.dat");
               freopen(datPath,"wb",dat_fp);
        
               fwrite(dbff,1,rmEntry.dataOffset,dat_fp);
               if(dbff2)  fwrite(dbff2,1,amtLeft2,dat_fp);
               
               freopen(datPath,"r+b",dat_fp);
               
               printf("Successfully removed animation entry\n%s\n\n",fname);
               
               delete [] bff;
               if(bff2) delete [] bff2;
               delete [] dbff;
               if(dbff2) delete [] dbff2;     
               fflush(idx_fp); fflush(dat_fp);
            }
            else {
                fputs("Failed to remove entry: entry not found\n",stderr); 
                fputs(fname,stderr);
                fputs("\n\n",stderr);
            }       
        }    
    
};   

class IDXF_Add: public CMenuItem{
    public:
        virtual void prompt(){printf("(Add function)\n"); CMenuItem::prompt();}
        virtual void execute(const char* fname){
            
            FILE *fp =fopen(fname,"rb");
            if(!fp){ 
                fputs("Failied to open file ",stderr);
                fputs(fname,stderr);
                fputs(" \n",stderr);
                return;
            }
            
    
            float scale=1.0;
            int fsize=getfilesize(fp);
            if(!fsize) return; //empty file
            char *bff =new char[fsize];
            fread(bff,fsize,1,fp);
            fclose(fp);
            
            /*add spaces at end*/
            char nfname[256]={0}; strcpy(nfname,fname);
            rewind(skl_df);
            int sfsize = getfilesize(skl_df);
            char *sbff = new char[sfsize];
            fread(sbff,1,sfsize,skl_df);
            char *pstr=NULL;
            for(int i=sfsize; i>0 && !pstr ; i--){
                if(!strnicmp(&sbff[i],fname,strlen(fname))) pstr = &sbff[i];
            }    
            if(pstr) strcpy(nfname,pstr);
            delete [] sbff;
            
            
            idx_Entry entry(strlen(nfname)+10,getfilesize(dat_fp),fsize,scale,(short)(*bff),(short)(bff[2]),nfname);
            fseek(idx_fp,0,SEEK_END);
            fseek(dat_fp,0,SEEK_END);
            entry.Write(idx_fp);
            incfiles();
            fwrite(bff,fsize,1,dat_fp);
            
            delete [] bff;
            fflush(idx_fp); fflush(dat_fp);
            printf("Added file successfully \n%s\n\n",nfname);

        }    
};

class IDXF_Replace: public CMenuItem{
    public:
        virtual void prompt(){printf("(Replace function)\n"); CMenuItem::prompt();}
        virtual void execute(const char*fname){
            IDXF_Remove rmeve;
            rmeve.execute(fname);
            
            IDXF_Add add;
            add.execute(fname);
            
            }
};

class IDXF_MovementManip: public CMenuItem{
    public:
        virtual void prompt(){printf("(Movement rate function)\n"); CMenuItem::prompt();}
        float transit(FILE* fp,unsigned int offset, int num,float scale=-1, bool print=false){
            if(!fp) return -1.0;
            fseek(fp,offset,SEEK_SET);
            
            float *transit = new float[num];
                      
            for(int i=0; i<num; i++){
                fseek(fp,8,SEEK_CUR);
                fread(&transit[i],4,1,fp);
            }    
            
            /*Calulate speed*/
            float sum=0.0;
            for(int i=0; i<num; i++) sum+=transit[i];
            float rate = sum/(((num*2)+1)/20.0f);
            if(print) printf("Speed: %4.2fm/s %4.2fkm/h %4.2fmph\n",rate,(rate*3.6f),rate*2.23694f );
            
           
            if(scale!=-1){
                
            for(int i=0; i<num; i++) transit[i]*=scale;
           
            /*Write new scale*/
            fseek(fp,offset,SEEK_SET);
            
            for(int i=0; i<num; i++){
                fseek(fp,8,SEEK_CUR);
                fwrite (&transit[i],4,1,fp);
                fflush(fp);
            }
         }    
            delete [] transit;
            return rate;
        }    
        
        virtual void execute(const char*str){
            float scale = -1.0f;
            char  scalestr[32]={0};
            char fname[256]={0};
            strncpy(fname,str,255);
            if(stristr(str,"--speed=")) sscanf(str,"--speed=%s %[^\r\n\0]",scalestr,fname);
            
            printf("Working on file %s\n",fname);
            FILE *fp = fopen(fname,"r+b");
            if(!fp){
                printf("Failed to open file\n");
                return;
            } 
            short header[2];
            fread(header,2,2,fp);   
            int pfsize=( (header[0]*header[1]*16) + (header[0]*12*2) + (((header[0]-1)/2)*12) + (((header[0]-1)/2)*4) +45);
            if(pfsize!=getfilesize(fp)){
                printf("File does not appear to be a valid cas file\n");
                fclose(fp);
                return;
            }   
            
           float rate = transit(fp,(header[0]*header[1]*16)+(header[0]*12)+5,((header[0]-1)/2),-1,true);          
                                  
            /*Get new scale*/
            
            while(scale==-1.0f){
                char tstr[256]={0};
                if(!(*scalestr)){
                printf("Enter new speed or ratio for example 0.75 or 3mph: ");
                gets(tstr);
                } 
                else strcpy(tstr,scalestr);
                   
                if(!sscanf(tstr,"%f",&scale) || scale<0.0f) scale =-1.0f;
                else if(stristr(tstr,"mph"))   scale =(scale/2.23694f)/rate;
                else if(stristr(tstr,"km"))    scale =(scale/3.6f)/rate;
                else if(stristr(tstr,"m/s"))   scale = scale/rate;
                
            }   
            printf("Scale:%f\n",scale); 
            
            /*Write first*/
            transit(fp,(header[0]*header[1]*16)+5,header[0],scale);
            /*Write second*/
            transit(fp,(header[0]*header[1]*16)+(header[0]*12)+5,((header[0]-1)/2),scale);
            /*Write third*/
            transit(fp,(header[0]*header[1]*16) + (header[0]*12) + (((header[0]-1)/2)*12) + (((header[0]-1)/2)*4)+5,header[0],scale);
            
        
            
            fclose(fp);

        }    
};


class IDXF_Help: public CMenuItem{
    public:
        virtual void prompt(){
           
            printf("(Help function)\n");
            printf("To modify pack.dat and pack.idx you must do so through basic functions\nChoose a function by entering the number or corresponding letter\n");
            printf("Valid functions:\n\t1) [A]dd new file\n\t2) [R]emove file\n\t3) [U]pdate file\n\t4) [H]elp\n\t5) [N]ew Skeleton\n\t6) [M]ovement speed\n\t7) [S]cale Skeleton\n\t8) [Q]uit\n"); 
             
                }
        virtual void execute(const char*){/*TODO*/}
};

class IDXF_NewSkel: public CMenuItem{
    public:
        IDXF_NewSkel(){tskl=NULL;}
        ~IDXF_NewSkel(){ if(tskl) delete [] tskl;}
        virtual void prompt(){
                    printf("(New Skeleton function)\n");
                    printf("Select a base skeleton from the following:\n\n");
                    
                    fseek(skl_if,16,SEEK_SET);
                    numSkels=0;
                    fread(&numSkels,4,1,skl_if);
                    tskl = new skl_Entry[numSkels];
                    
                    for(int i=0; i<numSkels; i++){
                        tskl[i].Read(skl_if);
                        printf("%s\n",tskl[i].fname);
                    }    
                    
                    CMenuItem::prompt("base skeleton");
                }
        virtual void execute(const char* baseSkel){
            int bskelnum=-1;
            for(int i=0; i<numSkels; i++) if(!stricmp(baseSkel,tskl[i].fname)) bskelnum=i; 
            
            if(bskelnum==-1){
                fputs("Error: base skeleton does not exist \"",stderr);
                fputs(baseSkel,stderr);
                fputs("\"\n",stderr);
                return;
            }    
                            
            printf("Enter new skeleton name: ");
            if(*gets(newSkelName)==NULL) return;
            
            skl_Entry newSkl(strlen(newSkelName)+1,getfilesize(skl_df),tskl[bskelnum].fsize, newSkelName);
            fseek(skl_if,0,SEEK_END);
            fseek(skl_df,tskl[bskelnum].offset,SEEK_SET);
            
            char *bff = new char[newSkl.fsize];
            fread(bff,1,newSkl.fsize,skl_df);
            fseek(skl_df,0,SEEK_END);
            
            bool useSame = false;
            char str[256]={0};
            printf("Do you want the skeleton to use the same animations as the base (no copy)? ");
            gets(str);
            if(*str=='y'  || *str=='Y') useSame=true;
            
            if(useSame==false){
                
            int amtWritten=0;
            char * porg = bff;
            char dirExist[512] = {0}; //strcpy(".\\AnimEditor.exe -l -a")
            while(amtWritten!=tskl[bskelnum].fsize){
                for(int i=0; i<tskl[bskelnum].fsize; i++){
                    if(!strnicmp(&bff[i],"data/",strlen("data/"))){
                    
                       int amt2write = (&bff[i] - (&bff[amtWritten])) + strlen("data/animations/");
                       amtWritten += fwrite(&bff[amtWritten],1,amt2write,skl_df);
                       fwrite(newSkelName,1,strlen(newSkelName),skl_df);
                       fwrite("/",1,1,skl_df);
                       newSkl.fsize+=strlen(newSkelName)+1;
                       
                       char command[256] = {0};
                       char fstr[256] ={0}; 
                       strncpy(fstr,&bff[i],strlen("data/animations/"));
                       strcat(fstr,newSkelName); strcat(fstr,"/"); 
                       
                       char dirtest[256] = {0}; strcpy(dirtest,fstr); strcat(dirtest,&bff[i+ strlen("data/animations/")]);
                       if(strrchr(dirtest,'/')) *(strrchr(dirtest,'/')+1)=0; 
                       
                       if(!stristr(dirExist,dirtest)) {
                       sprintf(command,"mkdir \"%s\"",dirtest); 
                       system(command);
                       //strcat(dirExist,"\"");
                       strcat(dirExist,dirtest);         
                       strcat(dirExist,"* " );
                        
                       }    
                       
                       strcat(fstr,&bff[i+strlen("data/animations/")]);
                       
                       char ofname[256] ={0};
                       strcpy(ofname,&bff[i]);
                       //ofname[strlen(ofname)-4] = 0; 
                       
                       /*some filenames have spaces at the end which screws things up*/
                       //CleanTrailing(ofname);
                       
                       //CreateBlankFile(fstr); 
                       
                     //  sprintf(command,"copy \"%s*\" /b \"%s\"",ofname,fstr);
                      // for(int k=0; k<strlen(command); k++) if(command[k]=='/' && command[k+1]!= 'b') command[k]='\\';
                       
                     
                       if(!copy(ofname,fstr)) printf("Failed to copy file...%s\n",fstr);
                       else printf("Copied file successfully...%s\n",fstr);
     
                    
                  }
                    
                }    
                amtWritten += fwrite(&bff[amtWritten],1,tskl[bskelnum].fsize - amtWritten,skl_df);  
            }   
              
            printf("%s\n",dirExist);
            
            char batfname[64]; strcpy(batfname,newSkelName);
          
            strcpy(batfname,newSkelName);
            FILE *afp = fopen(strcat(batfname,"_UpdateAnims.bat"), "wb");
            if(afp) fprintf(afp,".\\AnimEditor.exe -l -u %s\r\n",dirExist);
            if(afp) fclose(afp);
          }  /*end copy files*/                     
          else{
              fwrite(bff,1,newSkl.fsize,skl_df);
          } 
             
          incfiles(true);
          fseek(skl_if,0,SEEK_END);
          newSkl.Write(skl_if);
          fflush(skl_if); fflush(skl_df);
            
          delete [] tskl; tskl=NULL;
          delete [] bff;
            
                  
                         
                        
            printf("Succesfully created new skeleton\n");
            }
         private:
             int numSkels;
             char newSkelName[256];
             skl_Entry* tskl;
};

class IDXF_Scale: public CMenuItem{
    public:
        IDXF_Scale(){tskl=NULL;}
        ~IDXF_Scale(){ if(tskl) delete [] tskl;}
        
        virtual void prompt(){
                    printf("(Skeleton scale function)\n");
                    printf("Select a skeleton from the following:\n\n");
                    
                    fseek(skl_if,16,SEEK_SET);
                    numSkels=0;
                    fread(&numSkels,4,1,skl_if);
                    tskl = new skl_Entry[numSkels];
                    
                    for(int i=0; i<numSkels; i++){
                        tskl[i].Read(skl_if);
                        printf("%s\n",tskl[i].fname);
                    }    
                    
                    CMenuItem::prompt("skeleton");
                }
                
        virtual void execute(const char* baseSkel){
            int bskelnum=-1;
            float scale = -1.0f;
            char tstr[256]={0};
            
            for(int i=0; i<numSkels; i++) if(!stricmp(baseSkel,tskl[i].fname)) bskelnum=i; 
            
            if(bskelnum==-1){
                fputs("Error: skeleton does not exist \"",stderr);
                fputs(baseSkel,stderr);
                fputs("\"\n",stderr);
                return;
            } 
            
            
            char *data = new char[tskl[bskelnum].fsize];
            fseek(skl_df,tskl[bskelnum].offset,SEEK_SET);
            fread(data,1,tskl[bskelnum].fsize,skl_df);
            
            printf("Current scale: %4.2f\n",*((float*)data));
            while(scale==-1.0f){
                printf("Enter new scale: ");
                gets(tstr);
                sscanf(tstr,"%f",&scale);
            }
            float oldscale = *((float*)data );
            *((float*)data)=scale;
            int nbones = *((short*)(data+4));
            float *bnpos = (float*)(data+16);
         
            for(int i=0; i<(nbones); i++){
                
                bnpos[0]*=scale/oldscale;
                bnpos[1]*=scale/oldscale;
                bnpos[2]*=scale/oldscale;
                bnpos+=6;  
            }   
            fseek(skl_df,tskl[bskelnum].offset,SEEK_SET);
            fwrite(data,1,tskl[bskelnum].fsize,skl_df);
            
            delete [] data;
            fflush(skl_df);
        }    
       private:
           int numSkels;
           skl_Entry* tskl;  
};

class CMenuCtrl {
    public:
        CMenuCtrl(){
            memset(dataFile,0,sizeof(dataFile));
            items[0] = new IDXF_Add();
            items[1] = new IDXF_Remove();
            items[2] = new IDXF_Replace();  
            items[3] = new IDXF_Help();
            items[4] = new IDXF_NewSkel(); 
            items[5] = new IDXF_MovementManip();
            items[6] = new IDXF_Scale();
        }    
        
        void call(char fname[],int fn){items[fn]->execute(fname);}
        
        void run(){
            int menuSel=-1;
            while((menuSel=prompt())!=MENU_QUIT){
                items[menuSel-1]->prompt();
                //clear();
            }    
        }    
        
    private:
        void clear(){}
        
        int  prompt(){
            int sel=-1;
            
            while(sel==-1){
                char inputStr[256];
                printf("AnmEdt: ");
              
                inputStr[0] = getch();
                if(inputStr[0]=='a' || inputStr[0]=='A') sel = 1;
                else if(inputStr[0]=='r'  || inputStr[0]=='R')  sel = 2;
                else if(inputStr[0]=='u'  || inputStr[0]=='U')  sel = 3;
                else if(inputStr[0]=='h'  || inputStr[0]=='H')  sel = 4;
                else if(inputStr[0]=='\n' || inputStr[0]=='\r') sel = 4;
                else if(inputStr[0]=='n'  || inputStr[0]=='N')  sel = 5;
                else if(inputStr[0]=='m'  || inputStr[0]=='M')  sel = 6;
                else if(inputStr[0]=='s'  || inputStr[0]=='S')  sel = 7;
                else if(inputStr[0]=='q'  || inputStr[0]=='Q')  sel = 8;
                else sscanf(inputStr,"%d",&sel);
                
                if(sel>NUMITEMS+1 || sel<1) {
                    sel = -1;
                    printf("Invalid command!\n");
                }    
            } 
            return sel;   
        }  
          
    private:
        CMenuItem* items[NUMITEMS];
        char dataFile[256];        
        
};   
  


int main(int argc, char **argv)
{
    //getcwd(pcwd,_MAX_PATH);
    strcpy(pcwd,argv[0]); *strrchr(pcwd,'\\')='\0'; 
    
    printf("Animation editor\nVersion 0.51b\nA Trivium project\nMade by Vercingetorix\n\nType \"h\" for help\n\n");
    
    if(!idx_fp || !dat_fp || !skl_if || !skl_df) 
        printf("Error opening data file.\nMake sure you run this program in the Rome - Total War\\data\\animation folder\n");
    else{
        CMenuCtrl menctrl;
        if(argc>1){
           bool list=false;
           for(int i=1; i<argc; i++){
               int k=i;
               if(!strcmp(argv[i],"-l")) list=true;       
               else if (!strcmp(argv[i],"-a")) while(i<(list ? argc-1 : k+1) ) menctrl.call(argv[++i],0); 
               else if (!strcmp(argv[i],"-r")) while(i<(list ? argc-1 : k+1) ) menctrl.call(argv[++i],1); 
               else if (!strcmp(argv[i],"-u")) while(i<(list ? argc-1 : k+1) ) menctrl.call(argv[++i],2); 
               else if (!strcmp(argv[i],"-m")) while(i<(list ? argc-1 : k+1) ) menctrl.call(argv[++i],5); 
           }
       }        
       else
       menctrl.run();
    }    
    system("PAUSE");

  return 0;
}

